/********************************************************************
 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company's customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

#include <xc.h>
#include "system.h"
#include "system_config.h"
#include "usb/usb.h"
#include "usb/usb_device_cdc.h"

#include "patterns.h"

#define min(a,b) ((a<b)? a:b)

#define MAX_BUFFER 176
#define MAX_TRANSFER 35

char buffer[MAX_BUFFER];
uint8_t head = 0;
uint8_t newTail = 0;
uint8_t tail = 0;

void doWork( void ) {
    
    // if USB idle, free up space from last transfer
    if ((USBUSARTIsTxTrfReady()) && (newTail != tail)) {
        tail = newTail;
    }

    // if USB idle, and we have stuff to send
    uint8_t len;
    if ((USBUSARTIsTxTrfReady()) && (head != tail)) {
        if (head > tail) {
            len = min(head-tail,MAX_TRANSFER);
            putUSBUSART(&buffer[tail],len);
            newTail += len;
        } else {
            len = min(MAX_BUFFER-tail,MAX_TRANSFER);
            putUSBUSART(&buffer[tail],len);
            newTail = (newTail + len) % MAX_BUFFER;
        }
    }
}

//
// overflow: normalayl I would dump the old data, but the oldest
// data might have a pointer that was given to the USB send code.
// To make my life easy Im dumping the newset data.
//

void debugOut(char* debugString) {
    uint8_t size = strlen(debugString);
    uint8_t i;
    
    for (i=0;i<size;i++) {
        if (head+1 != tail) { // toss. buffer full
            buffer[head++] = debugString[i];
            head %= MAX_BUFFER;
        } 
    }
}

void debugPackedOut(uint16_t address, uint8_t size) {
    uint8_t i;
    
    for (i=0;i<size;i++) {
        if (head+1 != tail) { // toss. buffer full
            buffer[head++] = (i % 2)? packedRead(address+(i>>1),0) : packedRead(address+(i>>1),1);
            head %= MAX_BUFFER;
        } 
    }
}
