/********************************************************************
 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company's customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

/** INCLUDES *******************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "system.h"
#include "system_config.h"

#include "usb/usb.h"
#include "usb/usb_device_cdc.h"
#include "usb/app_device_cdc_basic.h"

//#include "usb/app_device_custom_cdc_basic.h"
//#include "debug.h"
#include "patterns.h"
#include "ascii_table.h"
#include "main.h"

char debugString[40];

uint8_t p_intensity=0;
uint8_t run = 1;
uint8_t demo_mode = 0;
uint8_t user_id = 0;
uint8_t plockout[4];
uint8_t pattern_speed=0;
uint8_t old_button = 0;

void update_pattern(void);
void handle_push_button(void);

extern void doWork(void);

extern uint8_t * fDemoMode;
extern uint8_t * fUserMsgSize;
extern uint8_t * fUserId;

MAIN_RETURN main(void)
{
    int delay=0;
//  start DDF

//    ANSELA = 0x00;
    ANSELC = 0x00;
    LATA = 0x00;
    LATC = 0x00;
    TRISA  = 0x00; // trying to get A4 and A5 to work
    TRISC  = 0x00;

    OSCTUNE = 0;
    OSCCON = 0x7C;   // PLL enabled, 3x, 16MHz internal osc, SCS external
    OSCCONbits.SPLLMULT = 1;   // 1=3x, 0=4x
    ACTCON = 0x90;   // Clock recovery on, Clock Recovery enabled; SOF packet
    OPTION_REGbits.nWPUEN = 0;  //enable weak pullups.
    WPUAbits.WPUA3 = 1; // weak pullup on A3
    WPUAbits.WPUA4 = 0;
    WPUAbits.WPUA5 = 0;
    
    demo_mode = packedRead((uint16_t) &fDemoMode,0);
    user_msg_size = packedRead((uint16_t) &fUserMsgSize,0);
    user_id = packedRead((uint16_t) &fUserId,0);
//    plockout[0] = setting[3];
//    plockout[1] = setting[4];
//    plockout[2] = setting[5];
//    plockout[3] = setting[6];
    
    // holding button on powerup turns on demo mode.
    if (!sw0) {demo_mode = 0x80;}

    USBDeviceInit();
    USBDeviceAttach();
//    initPatternTable();

    //  Enable Timer0
    //           +------------------ 0 Weak Pullup enabled
    //           |+----------------- 1 edge (don'r care)
    //           ||+---------------- 0 instuction clock
    //           |||+--------------- 1 edge (don't care)
    //           ||||+-------------- 1 PSA assigned
    //           |||||+++----------- 1 prescale 1:2
    OPTION_REG = 0b01000000;
    INTCONbits.TMR0IE = 1; // enable timer0 interupt
       
    while(1) {
        SYSTEM_Tasks();
        APP_DeviceCDCBasicDemoTasks();
        doWork();
        doMenu();
        
        // Note: Other application specific actions can be placed here
        if ((wait_timer == 0) && (run == 1)) {
            wait_timer = 1;
            delay++;
#ifndef DEBUG
            if ((!sw0) || old_button) {
                handle_push_button();
            } else {
#endif
                if (delay > pattern_speed*16) {
                    delay = 0;
                    update_pattern();
                }
#ifndef DEBUG
            }
#endif
        }
    }//end while
}//end main

/*******************************************************************************
 End of File
*/