/********************************************************************
 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company's customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

/** INCLUDES *******************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "system.h"
#include "system_config.h"

#include "usb/usb.h"
#include "usb/usb_device_cdc.h"
#include "usb/app_device_cdc_basic.h"

//#include "usb/app_device_custom_cdc_basic.h"
//#include "debug.h"
#include "patterns.h"
#include "ascii_table.h"

uint8_t up_down( uint8_t, uint8_t);
void add_and_shift( uint8_t);
void next_pattern(void);
void back_pattern(void);
inline void dotCode(void);
inline void processChars(void);

extern uint8_t pattern_speed;
extern uint8_t demo_mode;
extern uint8_t old_button;

uint8_t p_table=0;
uint8_t fader_cycle=0;
uint8_t cycle_count =0;
uint8_t demo_loops;
uint8_t led_data_save[5];

void update_pattern() {
    
    static uint8_t table_type=0;

    uint8_t i, j, a, b, x, y, z, l, temp_in;
    static uint8_t temp_red[8], char_in, step, delay2;
    static char buffer[20];
   
// pull extra info bytes off the front of the pattern;

    if (topOfPattern()) {
        table_type = get_next_pattern_byte( p_table );
        pattern_speed = get_next_pattern_byte( p_table );
        step = 0;
    }

    switch (table_type) {
        case 8:
        case 9:
            if (fader_cycle == 0) {
                if (table_type == 9) {srand((unsigned)TMR0);}
                for (i=0;i<8;i++) {
                    if (table_type == 8) {
                        temp_red[i] = get_next_pattern_byte(p_table );
                    } else if (table_type == 9) {
                        temp_red[i] = rand();
                    }
                }
                cycle_count = get_next_pattern_byte(p_table );
            }
            for (i=0;i<8;i++) {
                LEDS[i][0] = up_down((temp_red[i] & 0x80),LEDS[i][0]);
                LEDS[i][1] = up_down((temp_red[i] & 0x40),LEDS[i][1]);
                LEDS[i][2] = up_down((temp_red[i] & 0x20),LEDS[i][2]);
                LEDS[i][3] = up_down((temp_red[i] & 0x10),LEDS[i][3]);
                LEDS[i][4] = up_down((temp_red[i] & 0x08),LEDS[i][4]);
                LEDS[i][5] = up_down((temp_red[i] & 0x04),LEDS[i][5]);
                LEDS[i][6] = up_down((temp_red[i] & 0x02),LEDS[i][6]);
                LEDS[i][7] = up_down((temp_red[i] & 0x01),LEDS[i][7]);
            }
            fader_cycle = (fader_cycle < cycle_count) ? (fader_cycle+1) : (0);
            break;

        case 3:
            delay2++;
            if (delay2 > 6) {
                processChars();
                delay2 = 0;
            }
            break;
        case 6:
            add_and_shift( get_next_pattern_byte(p_table));
            break;
        case 7:
            dotCode();
            break;
    }

    if (pattern_done(p_table)) {
        demo_loops = demo_loops - 1;
        if ((demo_loops == 0) && (demo_mode == 0x80)) {
            next_pattern();
        }
    }
}

inline void processChars(void) {
    static uint8_t step = 0;
    static uint8_t char_in;
    static uint16_t pCharData;
    static uint8_t charLines;
    
    if (step==0) {
        char_in = get_next_pattern_byte(p_table);
            while (char_in < 20) {
                char_in = get_next_pattern_byte(p_table);
            }
        add_and_shift( 0x00);
        step = 1;
        pCharData = ((uint16_t) &chardata)+(char_in - 0x20)*3;
        charLines = packedRead(pCharData, true);
    } else {            
        add_and_shift( packedRead(pCharData + (step>>1), ((step % 2)==0)) ); 
        step = (step>=charLines)? 0 : step+1;
    }
}

inline void dotCode(void) {
    uint8_t i, j, a, x, y, char_in;
    bool eof;
       
    do {
        char_in = get_next_pattern_byte(p_table);
        y = (char_in & 0b01110000)>>4;
        x = (char_in & 0b00001110)>>1;
        a = (char_in & 0b00000001);
        eof = false;
        if (y == 0) {
            switch ((x<<1)+a) {
                case 0:  // clear all
                case 1:  // set all
                    for (i=0;i<8;i++) {
                        for (j=0;j<8;j++) {
                           LEDS[i][j] = a;
                        }
                    }
                    break;
                case 2: // shift down fill 0
                case 3: // shift down fill 1
                    for (i=0;i<8;i++) {
                        for (j=1;j<7;j++) {
                            LEDS[i][j] =   LEDS[i][j+1];
                        }
                    }
                    for (i=0;i<8;i++) {
                        LEDS[i][7] = a;
                    }
                    break;
                case 6: // shift right fill 0
                case 7: // shift right fill 1
                    for (i=7;i>0;i--) {
                        for (j=1;j<8;j++) {
                           LEDS[i][j] =   LEDS[i-1][j];
                        }
                    }
                    for (i=1;i<8;i++) {
                        LEDS[0][i] = a;
                    }
                    break;
                case 15: // End Of Frame
                    eof=true;
                    break;
                default:
                    break;
            }
        } else {
            LEDS[x][y] = a;
        }
    } while (eof==false);
}

void add_and_shift( uint8_t in) {

    uint8_t i,j;
   
    for (i=0;i<7;i++) {
        for (j=1;j<=7;j++) {
            LEDS[i][j] =   LEDS[i+1][j];
        }
    }

    LEDS[7][7]   = (in   & 0x40) ? (0x07) : (0x00);
    LEDS[7][6]   = (in   & 0x20) ? (0x07) : (0x00);
    LEDS[7][5]   = (in   & 0x10) ? (0x07) : (0x00);
    LEDS[7][4]   = (in   & 0x08) ? (0x07) : (0x00);
    LEDS[7][3]   = (in   & 0x04) ? (0x07) : (0x00);
    LEDS[7][2]   = (in   & 0x02) ? (0x07) : (0x00);
    LEDS[7][1]   = (in   & 0x01) ? (0x07) : (0x00);
}

void handle_push_button() {

    unsigned char i, temp;
    int delay_count;
    
    if (old_button == 0) {
        for (i=0;i<4;i++) {
            led_data_save[i] = LEDS[0][7-i];      // save the 4 LED states
            LEDS[0][7-i] = 0x00;

        }
        led_data_save[4] = LEDS[0][7];
        LEDS[0][7] = 0x00;

    }

    temp = p_table+1;

    LEDS[1][7] = (demo_mode == 0x80) ? (0xff) : (0x00);  // display the table number
    LEDS[0][7] = (temp & 0x01) ? (0xff) : (0x00);  // display the table number
    LEDS[0][6] = (temp & 0x02) ? (0xff) : (0x00);
    LEDS[0][5] = (temp & 0x04) ? (0xff) : (0x00);
    LEDS[0][4] = (temp & 0x08) ? (0xff) : (0x00);

    old_button++;

#define BUTTON_DELAY 250

#ifndef DEBUG
    if (!sw0) {
        delay_count = 3600;
        do {
            delay_count--;
        } while(delay_count);

        if (old_button > BUTTON_DELAY) {
            next_pattern();
            old_button = 1;
        }
    } else {
        for (i=0;i++;i<4) {
            LEDS[0][7-i] = led_data_save[i];  // no button, restore the 4 LEDs state.
        }
        LEDS[1][7] = led_data_save[4];  // no button, restore the 4 LEDs state.
        old_button = 0;
//      sprintf(debugString, "pattern %d %d", p_table, demo_mode);
//	  debugOut(debugString);
    }
#endif
}

void next_pattern(void) {
    p_table = (p_table >= USER_DEFINED_PATTERN-1) ? (0):(p_table+1);
    setTopOfPattern();
    demo_loops = patterns[p_table].loops;
}

void back_pattern(void) {
    p_table = (p_table < 1) ? (USER_DEFINED_PATTERN-1):(p_table-1);
    setTopOfPattern();
    demo_loops = patterns[p_table].loops;
}

void set_pattern(uint8_t id) {
    p_table = id % USER_DEFINED_PATTERN;
    setTopOfPattern();
    demo_loops = patterns[p_table].loops;
}

#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

uint8_t up_down( uint8_t change, uint8_t data) {

    if (change) {
        return(MIN(8,data+1));
    } else {
        return(MAX(0,(signed char) data-1));
    }
}
/*******************************************************************************
 End of File
*/