/********************************************************************
 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company's customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

#include <xc.h>
#include "system.h"
#include "system_config.h"
#include "usb/usb.h"

/** CONFIGURATION Bits **********************************************/
// PIC 16F1459 fuse configuration:
// Config word 1 (Oscillator configuration)
// 20Mhz crystal input scaled to 48Mhz and configured for USB operation
#if defined (USE_INTERNAL_OSC)
    // CONFIG1
    #pragma config FOSC = INTOSC    // Oscillator Selection Bits (INTOSC oscillator: I/O function on CLKIN pin)
    #pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
    #pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
    #pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
    #pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
    #pragma config BOREN = OFF      // Brown-out Reset Enable (Brown-out Reset disabled)
    #pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
    #pragma config IESO = OFF       // Internal/External Switchover Mode (Internal/External Switchover Mode is disabled)
    #pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

    // CONFIG2
    #pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
    #pragma config CPUDIV = NOCLKDIV// CPU System Clock Selection Bit (NO CPU system divide)
    #pragma config USBLSCLK = 48MHz // USB Low SPeed Clock Selection bit (System clock expects 48 MHz, FS/LS USB CLKENs divide-by is set to 8.)
    #pragma config PLLMULT = 3x     // PLL Multipler Selection Bit (3x Output Frequency Selected)
    #pragma config PLLEN = ENABLED  // PLL Enable Bit (3x or 4x PLL Enabled)
    #pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
    #pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
    #pragma config LPBOR = OFF      // Low-Power Brown Out Reset (Low-Power BOR is disabled)
    #pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)
#else
    // CONFIG1
    #pragma config FOSC = HS        // Oscillator Selection Bits (HS Oscillator, High-speed crystal/resonator connected between OSC1 and OSC2 pins)
    #pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
    #pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
    #pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
    #pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
    #pragma config BOREN = OFF      // Brown-out Reset Enable (Brown-out Reset disabled)
    #pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
    #pragma config IESO = OFF       // Internal/External Switchover Mode (Internal/External Switchover Mode is disabled)
    #pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

    // CONFIG2
    #pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
    #pragma config CPUDIV = NOCLKDIV// CPU System Clock Selection Bit (NO CPU system divide)
    #pragma config USBLSCLK = 48MHz // USB Low SPeed Clock Selection bit (System clock expects 48 MHz, FS/LS USB CLKENs divide-by is set to 8.)
    #pragma config PLLMULT = 4x     // PLL Multipler Selection Bit (4x Output Frequency Selected)
    #pragma config PLLEN = ENABLED  // PLL Enable Bit (3x or 4x PLL Enabled)
    #pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
    #pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
    #pragma config LPBOR = OFF      // Low-Power Brown Out Reset (Low-Power BOR is disabled)
    #pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)
#endif
/*********************************************************************
* Function: void SYSTEM_Initialize( SYSTEM_STATE state )
*
* Overview: Initializes the system.
*
* PreCondition: None
*
* Input:  SYSTEM_STATE - the state to initialize the system into
*
* Output: None
*
********************************************************************/
void SYSTEM_Initialize( SYSTEM_STATE state )
{
//    switch(state)
//    {
//        case SYSTEM_STATE_USB_START:
//            LED_Enable(LED_USB_DEVICE_STATE);
//            LED_Enable(LED_USB_DEVICE_HID_CUSTOM);
            
//            BUTTON_Enable(BUTTON_USB_DEVICE_HID_CUSTOM);
            
 //           ADC_SetConfiguration(ADC_CONFIGURATION_DEFAULT);
//            ADC_Enable(ADC_CHANNEL_POTENTIOMETER);
//            break;
            
//        case SYSTEM_STATE_USB_SUSPEND: 
//            break;
            
//        case SYSTEM_STATE_USB_RESUME:
//            break;
//    }
}

uint8_t strobe =0;
uint8_t wait_timer = 1;
uint8_t LEDS[8][8];
			
void interrupt SYS_InterruptHigh(void)
{
    uint8_t Ax_dir=0,Cx_dir=0;

    #if defined(USB_INTERRUPT)
        USBDeviceTasks();
    #endif

    if (INTCONbits.TMR0IF){  // Interrupt Check
       INTCONbits.TMR0IF = 0;

       strobe = ((strobe+1) & 0x07);

       switch(strobe)
    {
	    case 0:
                if (LEDS[1][7]<=0)  Ax_dir = Ax_dir | 0b00100000; // A5
                if (LEDS[1][1]<=0)  Cx_dir = Cx_dir | 0b00000001; // C0
                if (LEDS[1][2]<=0)  Cx_dir = Cx_dir | 0b00000010; // C1
                if (LEDS[1][3]<=0)  Cx_dir = Cx_dir | 0b00000100; // C2
                if (LEDS[1][4]<=0)  Cx_dir = Cx_dir | 0b00001000; // C3
                if (LEDS[1][5]<=0)  Cx_dir = Cx_dir | 0b00010000; // C4
                if (LEDS[1][6]<=0)  Cx_dir = Cx_dir | 0b00100000; // C5

                LATC = 0b00000000;
                TRISA = Ax_dir;
                TRISC = Cx_dir;
		LATA = 0b00010000; // A4
		LATC = 0b00000000;
		    break;
           case 1:
                if (LEDS[0][7]<=0)  Ax_dir = Ax_dir | 0b00010000; // A4
                if (LEDS[0][1]<=0)  Cx_dir = Cx_dir | 0b00000001; // C0
                if (LEDS[0][2]<=0)  Cx_dir = Cx_dir | 0b00000010; // C1
                if (LEDS[0][3]<=0)  Cx_dir = Cx_dir | 0b00000100; // C2
                if (LEDS[0][4]<=0)  Cx_dir = Cx_dir | 0b00001000; // C3
                if (LEDS[0][5]<=0)  Cx_dir = Cx_dir | 0b00010000; // C4
                if (LEDS[0][6]<=0)  Cx_dir = Cx_dir | 0b00100000; // C5

                LATA = 0b00000000;
                TRISA = Ax_dir;
                TRISC = Cx_dir;
		LATA = 0b00100000; // A5
		LATC = 0b00000000;
		    break;
           case 2:
                if (LEDS[7][2]<=0)  Ax_dir = Ax_dir | 0b00010000; // A4
                if (LEDS[7][1]<=0)  Ax_dir = Ax_dir | 0b00100000; // A5
                if (LEDS[7][7]<=0)  Cx_dir = Cx_dir | 0b00000010; // C1
                if (LEDS[7][6]<=0)  Cx_dir = Cx_dir | 0b00000100; // C2
                if (LEDS[7][5]<=0)  Cx_dir = Cx_dir | 0b00001000; // C3
                if (LEDS[7][4]<=0)  Cx_dir = Cx_dir | 0b00010000; // C4
                if (LEDS[7][3]<=0)  Cx_dir = Cx_dir | 0b00100000; // C5

                LATA = 0b00000000;
                TRISA = Ax_dir;
                TRISC = Cx_dir;
		LATA = 0b00000000;
		LATC = 0b00000001; // C0
	            break;
           case 3:
                if (LEDS[6][2]<=0)  Ax_dir = Ax_dir | 0b00010000; // A4
                if (LEDS[6][1]<=0)  Ax_dir = Ax_dir | 0b00100000; // A5
                if (LEDS[6][7]<=0)  Cx_dir = Cx_dir | 0b00000001; // C0
                if (LEDS[6][6]<=0)  Cx_dir = Cx_dir | 0b00000100; // C2
                if (LEDS[6][5]<=0)  Cx_dir = Cx_dir | 0b00001000; // C3
                if (LEDS[6][4]<=0)  Cx_dir = Cx_dir | 0b00010000; // C4
                if (LEDS[6][3]<=0)  Cx_dir = Cx_dir | 0b00100000; // C5

                LATC = 0b00000000;
                TRISA = Ax_dir;
                TRISC = Cx_dir;
		LATA = 0b00000000;
		LATC = 0b00000010; // C1
	            break;
           case 4:
                if (LEDS[5][2]<=0)  Ax_dir = Ax_dir | 0b00010000; // A4
                if (LEDS[5][1]<=0)  Ax_dir = Ax_dir | 0b00100000; // A5
                if (LEDS[5][7]<=0)  Cx_dir = Cx_dir | 0b00000001; // C0
                if (LEDS[5][6]<=0)  Cx_dir = Cx_dir | 0b00000010; // C1
                if (LEDS[5][5]<=0)  Cx_dir = Cx_dir | 0b00001000; // C3
                if (LEDS[5][4]<=0)  Cx_dir = Cx_dir | 0b00010000; // C4
                if (LEDS[5][3]<=0)  Cx_dir = Cx_dir | 0b00100000; // C5

                LATC = 0b00000000;
                TRISA = Ax_dir;
                TRISC = Cx_dir;
		LATA = 0b00000000;
		LATC = 0b00000100; // C2
	            break;
           case 5:
                if (LEDS[4][2]<=0)  Ax_dir = Ax_dir | 0b00010000; // A4
                if (LEDS[4][1]<=0)  Ax_dir = Ax_dir | 0b00100000; // A5
                if (LEDS[4][7]<=0)  Cx_dir = Cx_dir | 0b00000001; // C0
                if (LEDS[4][6]<=0)  Cx_dir = Cx_dir | 0b00000010; // C1
                if (LEDS[4][5]<=0)  Cx_dir = Cx_dir | 0b00000100; // C2
                if (LEDS[4][4]<=0)  Cx_dir = Cx_dir | 0b00010000; // C4
                if (LEDS[4][3]<=0)  Cx_dir = Cx_dir | 0b00100000; // C5

                LATC = 0b00000000;
                TRISA = Ax_dir;
                TRISC = Cx_dir;
		LATA = 0b00000000;
		LATC = 0b00001000; // C3
		    break;
           case 6:
                if (LEDS[3][2]<=0)  Ax_dir = Ax_dir | 0b00010000; // A4
                if (LEDS[3][1]<=0)  Ax_dir = Ax_dir | 0b00100000; // A5
                if (LEDS[3][7]<=0)  Cx_dir = Cx_dir | 0b00000001; // C0
                if (LEDS[3][6]<=0)  Cx_dir = Cx_dir | 0b00000010; // C1
                if (LEDS[3][5]<=0)  Cx_dir = Cx_dir | 0b00000100; // C2
                if (LEDS[3][4]<=0)  Cx_dir = Cx_dir | 0b00001000; // C3
                if (LEDS[3][3]<=0)  Cx_dir = Cx_dir | 0b00100000; // C5

                LATC = 0b00000000;
                TRISA = Ax_dir;
                TRISC = Cx_dir;
		LATA = 0b00000000;
		LATC = 0b00010000; // C4
		    break;
           case 7:
                if (LEDS[2][6]<=0)  Ax_dir |= 0b00010000; // A4
                if (LEDS[2][7]<=0)  Ax_dir |= 0b00100000; // A5
                if (LEDS[2][1]<=0)  Cx_dir |= 0b00000001; // C0
                if (LEDS[2][2]<=0)  Cx_dir |= 0b00000010; // C1
                if (LEDS[2][3]<=0)  Cx_dir |= 0b00000100; // C2
                if (LEDS[2][4]<=0)  Cx_dir |= 0b00001000; // C3
                if (LEDS[2][5]<=0)  Cx_dir |= 0b00010000; // C4

                LATC = 0b00000000;
                TRISA = Ax_dir;
                TRISC = Cx_dir;
		LATA = 0b00000000;
		LATC = 0b00100000; // C5
                        wait_timer = 0;
	            break;

   }

    }
}
